/*
 * irreco - Ir Remote Control
 * Copyright (C) 2008 Joni Kokko (t5kojo01@students.oamk.fi)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "irreco_theme_manager.h"

 /**
 * @addtogroup IrrecoThemeManager
 * @ingroup Irreco
 *
 * Contains information of themes.
 *
 * @{
 */
 
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Prototypes                                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

void irreco_theme_manager_read_file_foreach(IrrecoDirForeachData * dir_data);

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Construction & Destruction                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * @name Construction & Destruction
 * @{
 */

IrrecoThemeManager *irreco_theme_manager_new(IrrecoData * irreco_data)
{
	IrrecoThemeManager *self;
	IRRECO_ENTER
	
	self = g_slice_new0(IrrecoThemeManager);

	self->irreco_data = irreco_data;

	self->themes =  irreco_string_table_new(
				(GDestroyNotify)irreco_theme_free, NULL);
	
	irreco_theme_manager_read_themes_from_dir(self, IRRECO_THEME_DIR);
	irreco_theme_manager_read_themes_from_dir(self, "/media/mmc1/irreco");
	irreco_theme_manager_read_themes_from_dir(self, "/media/mmc2/irreco");


	
	/* Attempt to add the style to the style array. */
	#if 0
	
	
	IRRECO_STRING_TABLE_FOREACH_DATA(self->themes, void *, theme) {
		IrrecoStringTable* buttons = irreco_theme_get_buttons(theme);
		IRRECO_STRING_TABLE_FOREACH_DATA(buttons, IrrecoThemeButton *,
						 button) {
		IrrecoButtonStyle *style;
		style = irreco_button_style_create(irreco_data,
						   button->name->str,
						   button->image_up->str,
						   button->image_down->str,
						   button->allow_text,
						   FALSE);

		style->text_format_up = g_strdup(button->text_format_up->str);
		style->text_format_down =
					g_strdup(button->text_format_down->str);
		style->text_padding = button->text_padding;

		if (!irreco_string_table_add(irreco_data->button_style_array,
					     style->name, style)) {
			IRRECO_ERROR("Dublicate style name \"%s\"\n",
				     style->name);
			irreco_button_style_destroy(style);
		}
		
		}
		IRRECO_STRING_TABLE_FOREACH_END
		}
	IRRECO_STRING_TABLE_FOREACH_END

	#endif

	IRRECO_RETURN_PTR(self);
}

void irreco_theme_manager_free(IrrecoThemeManager *self)
{
	IRRECO_ENTER
			
	g_assert(self != NULL);
	
	irreco_string_table_free(self->themes);
	self->themes = NULL;
	
	g_slice_free(IrrecoThemeManager, self);
	self = NULL;

	IRRECO_RETURN
}

/** @} */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Private Functions                                                          */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * @name Private Functions
 * @{
 */

void irreco_theme_manager_read_file_foreach(IrrecoDirForeachData * dir_data)
{
	IrrecoThemeManager *self = (IrrecoThemeManager*) dir_data->user_data_1;
	IrrecoTheme	*theme;
	IrrecoKeyFile	*keyfile;
	char *theme_name = NULL;
	GString	*conf = g_string_new(dir_data->filepath);
	IRRECO_ENTER

	g_string_append_printf(conf, "/theme.conf");
	keyfile = irreco_keyfile_create(dir_data->filepath,
					conf->str,
					"theme");
	if (keyfile == NULL) goto end;

	/* Required fields. */
	if (!irreco_keyfile_get_str(keyfile, "name", &theme_name)) {
		IRRECO_PRINTF("Could not read theme \"%s\"\n",
			      dir_data->filename);
		goto end;
	}

	theme = irreco_theme_new(theme_name, dir_data->filepath);

	irreco_string_table_add(self->themes, theme->name->str, theme);

	end:
	g_string_free(conf, TRUE);
	if (keyfile != NULL) {
		irreco_keyfile_destroy(keyfile);
	}

	if (theme_name != NULL) {
		g_free(theme_name);
	}
	irreco_theme_print(theme);

	IRRECO_RETURN
}

/** @} */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Public Functions                                                           */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * @name Public Functions
 * @{
 */

void irreco_theme_manager_print(IrrecoThemeManager *self)
{
	IRRECO_ENTER

	#if 1
	irreco_string_table_print(self->themes);
	#endif

	#if 0
	IRRECO_STRING_TABLE_FOREACH_DATA(self->themes, void *, theme)
		IRRECO_PRINTF("Theme: %s \n", theme->name->str);
		irreco_theme_print(theme);
	IRRECO_STRING_TABLE_FOREACH_END
	#endif

	IRRECO_RETURN
}

IrrecoStringTable *irreco_theme_manager_get_themes(IrrecoThemeManager *self)
{
	IRRECO_ENTER

	IRRECO_RETURN_PTR(self->themes);
}

IrrecoStringTable *irreco_theme_manager_get_buttons(IrrecoThemeManager *self,
						   const gchar *theme_name)
{
	IrrecoStringTable	*buttons	= NULL;
	IrrecoTheme		*theme		= NULL;
	IRRECO_ENTER

	IRRECO_PRINTF("theme_name: %s\n", theme_name);
	if (irreco_string_table_get(self->themes, theme_name,
				    (gpointer*) &theme)) {
		buttons = theme->buttons;
	}

	IRRECO_RETURN_PTR(buttons);
}

IrrecoStringTable *irreco_theme_manager_get_backgrounds(IrrecoThemeManager *self,
						       const gchar *theme_name)
{
	IrrecoStringTable	*backgrounds	= NULL;
	IrrecoTheme		*theme		= NULL;
	IRRECO_ENTER

	if (irreco_string_table_get(self->themes, theme_name,
				    (gpointer*)&theme)) {
		backgrounds = theme->backgrounds;
	}

	IRRECO_RETURN_PTR(backgrounds);
}

void irreco_theme_manager_read_themes_from_dir(IrrecoThemeManager *self,
					       const gchar *dir)
{
	IrrecoDirForeachData themes;
	IRRECO_ENTER

	IRRECO_DEBUG("THEME_DIR = %s\n", dir);
	themes.directory = dir;

	themes.filesuffix = "";
	themes.user_data_1 = self;

	irreco_dir_foreach(&themes, irreco_theme_manager_read_file_foreach);

	irreco_string_table_sort_abc(self->themes);
	IRRECO_RETURN
}

gboolean irreco_theme_manager_get_button_style(IrrecoThemeManager *self,
					       const gchar *style_name,
					       IrrecoButtonStyle **style)
{
	IrrecoThemeButton *button = NULL;
	gboolean rvalue = FALSE;
	IRRECO_ENTER
	IRRECO_STRING_TABLE_FOREACH_DATA(self->themes, void *, theme) {
		IrrecoStringTable* buttons = irreco_theme_get_buttons(theme);

		if (irreco_string_table_get(buttons, style_name,
					    (gpointer *) &button)) {
			*style = irreco_button_style_create(self->irreco_data,
							button->style_name->str,
							button->image_up->str,
							button->image_down->str,
							button->allow_text,
							FALSE);

			(*style)->text_format_up = NULL;
			(*style)->text_format_down = NULL;
			if (button->text_format_up->len > 0) {
				(*style)->text_format_up =
					g_strdup(button->text_format_up->str);
			}
			if (button->text_format_down->len > 0) {
				(*style)->text_format_down =
					g_strdup(button->text_format_down->str);
			}
			(*style)->text_padding = button->text_padding;
			rvalue = TRUE;
		}
	}
	IRRECO_STRING_TABLE_FOREACH_END
	IRRECO_RETURN_BOOL(rvalue);
}
/** @} */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Events and Callbacks                                                       */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/** @} */


